# Example 5 using R

# set width of output
options("width"=120, "digits"=6)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do linear combinations in gls

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer


################################################################################
#####               BEGIN EXAMPLE 5 DATA MANIUPLATION                      #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/21_PSQF7375_Longitudinal/PSQF7375_Longitudinal_Example5/"
filename = "SAS_Chapter5.sas7bdat"
setwd(dir=filesave)

# Import chapter 5 stacked data with labels
Example5 = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example5 = as.data.frame(Example5)
# Labels can be used with gls, so I will use them here
# Sort data by PersonID (needed for correct RCOV matrix)
Example5 = sort_asc(Example5,PersonID,wave)
# Center time at first occasion
Example5$time=Example5$wave-1
Example5 = apply_labels(Example5, time="time: Time in Study (0=1)")
# Make new variable for wave with reference=4 to match other programs
Example5$wave4=relevel(factor(Example5$wave), ref=4)


################################################################################
#####                   BEGIN EXAMPLE 5 MODELS                             #####
################################################################################

# Open output text file
txtStart(file=paste0(filesave,"R_Example5_Output.txt"))

print("Chapter 5 Example: Get Variance of Time Predictor")
describe(x=Example5$time)  
txtComment(" ") # insert blank space in output


print("R GLS Ch 5: Saturated Means, Unstructured Variance Model")
print("ANSWER KEY for both sides of the model")
SatUN = gls(data=Example5, method="REML", model=outcome~1+factor(wave4), 
            correlation=corSymm(form=~as.numeric(wave4)|PersonID),
            weights=varIdent(form=~1|wave4)) 
print("Show results using incorrect DDF, with total leftover variance")
print("Total variance per occasion is created using SD multiplier")
summary(SatUN); summary(SatUN)$sigma^2 
print("Show R and RCORR matrices for first person") 
R=getVarCov(SatUN, individual="1", type="marginal"); R
RCORR=corMatrix(SatUN$modelStruct$corStruct)[[4]]; RCORR 
print("Wave means and pairwise mean differences wiht incorrect DDF")
emmeans(ref_grid(SatUN), pairwise~wave4, adjust="none") # tried mode="df.error"
print("Error when trying to get Satterthwaite DDF, so had to switch to residual DDF")
lsmeans(SatUN, "wave4", mode="df.error")
print("F-test p-value now based on Satterthwaite DDF"); anova(SatUN)
txtComment(" ") # insert blank space in output


print("R LME Ch 5: Saturated Means, Random Intercept Model")
print("ANSWER KEY for means side only")
SatRI = lme(data=Example5, method="REML", outcome~1+factor(wave4), random=~1|PersonID)
print("Show results using incorrect DDF"); summary(SatRI)
print("Show G, R, and V matrices for first person") 
G=getVarCov(SatRI, individual="1", type="random.effects"); G
R=getVarCov(SatRI, individual="1", type="conditional"); R
V=getVarCov(SatRI, individual="1", type="marginal"); V
print("Wave means and pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(SatRI), pairwise~wave4, adjust="none"); anova(SatRI) 
txtComment(" ") # insert blank space in output

print("R LMER Eq 5.1: Saturated Means, Random Intercept Model")
print("ANSWER KEY for means side only")
SatRIr = lmer(data=Example5, REML=TRUE, formula=outcome~1+factor(wave4)+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL") 
summary(SatRIr, ddf="Satterthwaite"); llikAIC(SatRIr, chkREML=FALSE)
print("Wave means, pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(SatRIr), pairwise~wave4, adjust="none"); anova(SatRIr)  
txtComment(" ") # insert blank space in output


print("R LME Eq 5.1: Empty Means, Random Intercept Model")
EmptyRI = lme(data=Example5, method="REML", outcome~1, random=~1|PersonID)
print("Show results using incorrect DDF"); summary(EmptyRI)  
print("Show G, R, and V matrices for first person") 
G=getVarCov(EmptyRI, individual="1", type="random.effects"); G
R=getVarCov(EmptyRI, individual="1", type="conditional"); R
V=getVarCov(EmptyRI, individual="1", type="marginal"); V
ICC=(V[[1]][2,1])/(V[[1]][1,1]); print("Show ICC"); ICC
txtComment(" ") # insert blank space in output

print("R LMER Eq 5.1: Empty Means, Random Intercept Model")
EmptyRIr = lmer(data=Example5, REML=TRUE, formula=outcome~1+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL") 
summary(EmptyRIr, ddf="Satterthwaite"); llikAIC(EmptyRIr, chkREML=FALSE)
txtComment(" ") # insert blank space in output


print("R LME Eq 5.3: Fixed Linear Time, Random Intercept Model")
FixLinRI = lme(data=Example5, method="REML", outcome~1+time, random=~1|PersonID)
print("Show results using incorrect DDF"); summary(FixLinRI)  
print("Show G, R, and V matrices for first person") 
G=getVarCov(FixLinRI, individual="1", type="random.effects"); G
R=getVarCov(FixLinRI, individual="1", type="conditional"); R
V=getVarCov(FixLinRI, individual="1", type="marginal"); V
ICC=(V[[1]][2,1])/(V[[1]][1,1]); print("Show ICC"); ICC
txtComment(" ") # insert blank space in output

print("R LMER Eq 5.3: Fixed Linear Time, Random Intercept Model")
FixLinRIr = lmer(data=Example5, REML=TRUE, formula=outcome~1+time+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL") 
summary(FixLinRIr, ddf="Satterthwaite"); llikAIC(FixLinRIr, chkREML=FALSE)
txtComment(" ") # insert blank space in output


print("R LME Eq 5.5: Random Linear Time Model")
RandLin = lme(data=Example5, method="REML", outcome~1+time, random=~1+time|PersonID)
print("Show results using incorrect DDF"); summary(RandLin)  
print("Show G, R, and V matrices for first person") 
G=getVarCov(RandLin, individual="1", type="random.effects"); G
R=getVarCov(RandLin, individual="1", type="conditional"); R
V=getVarCov(RandLin, individual="1", type="marginal"); V
txtComment(" ") # insert blank space in output

print("R LMER Eq 5.5: Random Linear Time Model")
RandLinr = lmer(data=Example5, REML=TRUE, formula=outcome~1+time+(1+time|PersonID))
print("Show results using Satterthwaite DDF including -2LL") 
summary(RandLinr, ddf="Satterthwaite"); llikAIC(RandLinr, chkREML=FALSE)
print("Does random linear time slope improve fit?")
ranova(RandLinr, reduce.term=TRUE) # Remove random slope and covariance
txtComment(" ") # insert blank space in output


print("R LME Random Linear Time Model + AR1 R Matrix")
RandLinAR1 = lme(data=Example5, method="REML", outcome~1+time, random=~1+time|PersonID,
              correlation=(corAR1(form=~as.numeric(time)|PersonID)))
print("Show results using incorrect DDF"); summary(RandLinAR1)  
print("Show G, R, and V matrices for first person") 
G=getVarCov(RandLinAR1, individual="1", type="random.effects"); G
R=getVarCov(RandLinAR1, individual="1", type="conditional"); R
V=getVarCov(RandLinAR1, individual="1", type="marginal"); V
print("Does AR1 residual correlation improve fit?")
anova(RandLinAR1,RandLin) # anova compares using LME versions
txtComment(" ") # insert blank space in output


# Toeplitz is not a pre-defined structure in R LME

# Close output text file
txtStop()

